from a2a.server.events import EventQueue


from a2a_acl.agent.acl_agent import ACLAgentExecutor
from a2a_acl.content_codecs.common import atom_codec_id

from a2a_acl.protocol_extension.acl_message import ACLMessage

from a2a_acl.interface import asi_parser
from a2a_acl.interface.interface import ACLAgentCard


my_card = asi_parser.read_file("failing.asi").add_codecs([atom_codec_id])


class PingableAgentExecutor(ACLAgentExecutor):

    def __init__(self, card: ACLAgentCard, url):
        super().__init__(agentcard=card, my_url=url)
        print(
            "Warning: launching a pure python A2A agent, unable to check .asi interface against python body."
        )

    async def execute_achieve(
        self,
        m: ACLMessage,
        output_event_queue: EventQueue,
    ):
        print("(Received a message: " + str(m) + ")")
        match m.content:
            case "ping":
                print("Received a ping from " + m.sender)
                raise Exception("bad")
