import os
import sys

sys.path.insert(0, os.path.abspath(os.path.join(os.path.dirname(__file__), "../..")))

advisor_port = 9989

# Change this flag to use the LLM instead of a mock
mock_flag = True
MOCK_FILE = "mock.txt"
# MOCK_FILE = "mock_error.txt"

# Change this to change the LLM
llm_model = "mistral/mistral-small-latest"
# llm_model = "gpt-4o-mini"

max_token = 100
