import context
from a2a_acl.agent.server_utils import run_server

from a2a_acl.utils.url import build_url

from reference_agent import ReferenceAgentExecutor, my_card

my_url = build_url(context.the_host, context.reference_agent_port)


async def main() -> None:
    run_server(
        ReferenceAgentExecutor(my_card, my_url),
        context.the_host,
        context.reference_agent_port,
    )


if __name__ == "__main__":
    import asyncio

    asyncio.run(main())
