from a2a.types import AgentExtension

A2A_ACL_PROTOCOL_URI = (
    "https://gitlab.eclipse.org/eclipse-research-labs/mosaico-project/a2a-acl/"
)

EXTENSION = AgentExtension(
    uri=A2A_ACL_PROTOCOL_URI, description="MOSAICO A2A ACL", required=True
)

MIME_TAG = "application/x-mosaico-a2a-acl"
